from typing import Any

email_pattern: Any
uri_pattern: Any

def validate_format(value: Any, format: Any) -> Any: ...
def unknown_format(value: Any) -> Any: ...
def validate_email(value: Any) -> Any: ...
def validate_uri(value: Any) -> Any: ...
