from typing import Any, Optional

def dereference(lookup_string: Any, struct: Any) -> Any: ...
def is_json_pointer(value: Any) -> Any: ...
def get_dicts(item: Any, dereference_using: Optional[Any] = ...) -> Any: ...
def get_strings(item: Any) -> Any: ...
