from typing import Any, Optional

from . import renderers as renderers

def get_swagger_view(
    title: Optional[Any] = ..., url: Optional[Any] = ..., patterns: Optional[Any] = ..., urlconf: Optional[Any] = ...
) -> Any: ...
