from typing import Any, Optional

def is_html_input(dictionary: Any) -> Any: ...
def parse_html_list(dictionary: Any, prefix: str = ..., default: Optional[Any] = ...) -> Any: ...
def parse_html_dict(dictionary: Any, prefix: str = ...) -> Any: ...
