from typing import Any, Optional

def manager_repr(value: Any) -> Any: ...
def smart_repr(value: Any) -> Any: ...
def field_repr(field: Any, force_many: bool = ...) -> Any: ...
def serializer_repr(serializer: Any, indent: Any, force_many: Optional[Any] = ...) -> Any: ...
def list_repr(serializer: Any, indent: Any) -> Any: ...
