from typing import Any

def link_sorting_key(link_item: Any) -> Any: ...
def get_links_from_document(node: Any, keys: Any = ...) -> Any: ...
def get_method(link: Any) -> Any: ...
def get_encoding(link: Any) -> Any: ...
def get_location(link: Any, field: Any) -> Any: ...
