from typing import Any

def datetime_formats(formats: Any) -> Any: ...
def date_formats(formats: Any) -> Any: ...
def time_formats(formats: Any) -> Any: ...
def humanize_strptime(format_string: Any) -> Any: ...
