from typing import Any, Optional

from openapi_codec import OpenAPICodec as _OpenAPICodec
from rest_framework.renderers import BaseRenderer

class OpenAPICodec(_OpenAPICodec):
    def encode(self, document: Any, **options: Any) -> Any: ...

class OpenAPIRenderer(BaseRenderer):
    media_type: str = ...
    charset: Any = ...
    format: str = ...
    def render(
        self, data: Any, accepted_media_type: Optional[Any] = ..., renderer_context: Optional[Any] = ...
    ) -> Any: ...
    def get_customizations(self) -> Any: ...

class SwaggerUIRenderer(BaseRenderer):
    media_type: str = ...
    format: str = ...
    template: str = ...
    charset: str = ...
    def render(
        self, data: Any, accepted_media_type: Optional[Any] = ..., renderer_context: Optional[Any] = ...
    ) -> Any: ...
    def set_context(self, data: Any, renderer_context: Any) -> None: ...
    def get_auth_urls(self) -> Any: ...
    def get_ui_settings(self) -> Any: ...
