from typing import Any, Optional, Tuple, Union

from django.db.models import QuerySet

def pygments_css(style: Any) -> Optional[str]: ...
def pygments_highlight(text: str, lang: str, style: Any) -> Any: ...
def md_filter_add_syntax_highlight(md: Any) -> bool: ...
def unicode_http_header(value: Union[str, bytes]) -> str: ...
def distinct(queryset: QuerySet[Any], base: Optional[QuerySet[Any]]) -> QuerySet[Any]: ...

SHORT_SEPARATORS: Tuple[str, str]
LONG_SEPARATORS: Tuple[str, str]
INDENT_SEPARATORS: Tuple[str, str]
